The Software Heritage License Blob Dataset - Replication Package
================================================================

This directory contains the software used to assemble the [Software Heritage License Blob Dataset][dataset].

How to run:

```sh
# Athena queries to generate the list of blobs to be included
# Run replication/01-select-blobs.sql on Athena, it takes ~35min

# Run 02-to-sha1.sql on Athena (2 minutes) and download the result (800MB),
# or do both at once with this command:
swh dataset athena query --database-name swh_20220425 --output-location s3://softwareheritage/tmp/license-dataset/to-sha1-20220425 replication/02-to-sha1.sql > license-blobs.csv

# Filter out files with \n character, remove useless quotes, and
# generate license-blobs.csv.zst
bash replication/03-clean.sh

# Optionally, decompress from the previous blob dump to avoid downloading them
# one by one:
tar -vxf ../2021-03-23/blobs.tar.zst

# Get (missing) blobs:
# (takes a few days when done incrementally after a year)
bash replication/04-fetch-blobs.sh

# Get stats on blobs (optional)
ls blobs/*/*/ | cat | grep "......................" | wc -l   # total number of blobs
du -hd0 blobs/  # total size of blobs

# Create the blob tarball
# (takes about a day)
tar -cv --sort=name blobs | zstdmt > blobs.tar.zst

# Generate sample blob tarball (blobs-sample20k.tar.zst)
bash replication/05-sample20k.sh

# Compute blobs-fileinfo.csv.zst
# (takes a few hours)
bash replication/05-stats.sh

# Compute blobs-scancode.csv.zst
# (takes about ten days; remove the call to `get_copyrights()` to make it
# go down to about three days if you don't need it)
pip3 install "scancode-toolkit==31.2.1"
bash replication/05-scancode.sh

# Extract previous result of blobs-origins.csv.zst
# (this step is optional; but if you omit it, then create an empty file named
# 'blobs-origins.old.csv'; and run time will be much longer)
zstdcat ../2021-03-23/blobs-origins.csv.zst > blobs-origins.old.csv

# Compute blobs-origins.csv.zst (should take about a week)
bash replication/05-swhid-to-origin.sh

# If there was any error running the previous command, run
# zstdcat blobs-origins.csv.zst >> blobs-origins.old.csv
# and re-run `bash replication/05-swhid-to-origin.sh` to retry missing ones.

# Compute blobs-nb-origins.csv.zst (should take about two weeks)
# Note: unlike blobs-origins.csv, blobs-nb-origins.old.csv must not be
# initialized with the content of last dataset; because counts may change.
touch blobs-nb-origins.old.csv
bash replication/05-swhid-to-nb-origin.sh

# If there was any error running the previous command, run
# zstdcat blobs-nb-origins.csv.zst >> blobs-nb-origins.old.csv
# and re-run `bash replication/05-swhid-to-nb-origin.sh` to retry missing ones.

# Compute blobs-earliest.csv.zst (takes 10 hours and half a TB of RAM)
# Assuming you are on swh1.enst.fr (not granet.internal.softwareheritage.org! there
# is not enough RAM for both this process and the main graph) and
# swh-graph-shm-mount.service is started:
nice -n 10 bash replication/05-earliest-revision.sh
```

6. earliest metadata: custom Java code available under `java/`

Note: files in the [publicly available dataset][dataset] were compressed using `zstdmt -19` instead of `zstdmt` to compressed them better.


[dataset]: https://annex.softwareheritage.org/public/dataset/license-blobs/2022-04-25/
[graphapi]: https://docs.softwareheritage.org/devel/swh-graph/api.html

