#!/usr/bin/env python3

# SPDX-FileCopyrightText: (C) 2022 Valentin Lorentz

# SPDX-License-Identifier: GPL-3.0-or-later

import os
import sys
import time

import requests

API_URL = "https://archive.softwareheritage.org/api/1"
API_URL = "http://graph.internal.softwareheritage.org:5009"


if len(sys.argv) == 2 and os.path.exists(sys.argv[1]):
    with open(sys.argv[1]) as fd:
        existing_counts = dict(
            line.strip().split("\t") for line in fd if line[-2] != "\t"
        )
elif len(sys.argv) == 1:
    existing_counts = {}
else:
    print(
        "This command takes no argument; SWHIDs should be passed as stdin.",
        file=sys.stderr
    )
    exit(1)

session = requests.Session()
session.headers["User-Agent"] = "SWH License Dataset (python/swh_count_origins.py)"


def retry(f, url):
    while True:
        try:
            return f(url)
        except requests.exceptions.ConnectionError:
            print(f"ConnectionError to {url} ; retrying", file=sys.stderr)
            time.sleep(60)
            continue

for line in sys.stdin:
    swhid = line.strip()
    if not swhid:
        continue
    if not swhid.startswith("swh:1:"):
        print(f"Invalid SWHID: {line}", file=sys.stderr)
        exit(1)

    count = existing_counts.get(swhid)

    if count:
        count = int(count)
    else:
        for wait_time in (0, 1, 2, 5, 10, 20, 30, 60):
            time.sleep(wait_time)
            url = ( 
                f"{API_URL}/graph/leaves/count/{swhid}"
                f"?direction=backward&return_types=ori"
            )
            resp = retry(session.get, url)
            if resp.status_code == 200:
                break
            else:
                print(file=sys.stderr)
                print(f"Status code: {resp.status_code} {url}", file=sys.stderr)
                continue
        else:
            # Failed multiple time
            print(file=sys.stderr)
            print(f"Giving up on {url}", file=sys.stderr)
            continue

            
        count = int(resp.content.decode("ascii").strip())

    print(f"{swhid}\t{count}")
